<?php
namespace Aelia\WC\TaxDisplayByCountry\Rules\Location_Selection;
if(!defined('ABSPATH')) { exit; } // Exit if accessed directly

use Aelia\WC\TaxDisplayByCountry\Definitions;

/**
 * Intercepts the "update checkout" event triggered by WooCommerce on the checkout
 * page, to update customer's country.
 *
 * @since x.x
 */
class Location_Selection_Checkout_Rule implements ILocation_Selection_Rule {
	/**
	 * Indicates if we are processing an Ajax call.
	 *
	 * @return bool
	 */
	protected static function doing_ajax(): bool {
		return defined('DOING_AJAX') && DOING_AJAX;
	}

	/**
	 * Indicates if the rule should be process.
	 *
	 * @param Location_Selection_Result $location_selection_result
	 * @param Location_Selection_Settings $settings
	 * @return boolean
	 */
	protected static function should_process_rule(Location_Selection_Result $location_selection_result, Location_Selection_Settings $settings): bool {
		return !empty($_POST[Definitions::ARG_CALC_SHIPPING]);
	}

	/**
	 * Processes this rule. The logic in this selector checks if the checkout data is being updated and
	 * tries to fetch the location from it.
	 *
	 * @param Location_Selection_Result $location_selection_result
	 * @return Location_Selection_Result
	 */
	public static function process_rule(Location_Selection_Result $location_selection_result, Location_Selection_Settings $settings): Location_Selection_Result {
		if(self::should_process_rule($location_selection_result, $settings)) {
			// Fetch the billing or shipping country and state, depending on the tax calculation settings
			if($settings->tax_based_on === 'shipping') {
				// Fetch the country
				if(isset($_POST[Definitions::ARG_CHECKOUT_SHIPPING_COUNTRY])) {
					$location_selection_result->country = wc_clean($_POST[Definitions::ARG_CHECKOUT_SHIPPING_COUNTRY]);
				}

				// Fetch the State/Province, if needed
				if($settings->should_handle_state && isset($_POST[Definitions::ARG_CHECKOUT_SHIPPING_STATE])) {
					$location_selection_result->state = wc_clean($_POST[Definitions::ARG_CHECKOUT_SHIPPING_STATE]);
				}
			}
			else {
				// Fetch the country
				if(isset($_POST[Definitions::ARG_BILLING_COUNTRY])) {
					$location_selection_result->country = wc_clean($_POST[Definitions::ARG_BILLING_COUNTRY]);
				}

				// Fetch the State/Province, if needed
				if($settings->should_handle_state && isset($_POST[Definitions::ARG_BILLING_STATE])) {
					$location_selection_result->state = wc_clean($_POST[Definitions::ARG_BILLING_STATE]);
				}
			}
			$location_selection_result->should_update_stored_data = true;
		}
		return $location_selection_result;
	}
}