<?php
namespace Aelia\WC\TaxDisplayByCountry;
if(!defined('ABSPATH')) { exit; } // Exit if accessed directly

use Aelia\WC\Integrations\Freemius\Freemius_Plugin_Integration;

/**
 * Implements a class that will render the settings page.
 */
class Settings_Renderer extends \Aelia\WC\Settings_Renderer {
	// @var string The URL to the support portal.
	const SUPPORT_URL = 'https://aelia.freshdesk.com/support/home';
	// @var string The URL to the contact form for general enquiries.
	const CONTACT_URL = 'https://aelia.co/contact/';

	/*** Settings Tabs ***/
	const TAB_TAX_DISPLAY = 'tax_display';
	const TAB_TAX_CALCULATION = 'tax_calculation';
	const TAB_SUPPORT = 'support';
	// @since 1.17.0.210511
	const TAB_LICENSE = 'license';

	// @since 1.18.0.220401
	const TAB_COUNTRY_SELECTION = 'country_selection';

	/*** Settings sections ***/
	const SECTION_TAX_DISPLAY = 'tax_display';
	const SECTION_TAX_CALCULATION = 'tax_calculation';
	const SECTION_USER_INTERFACE = 'tax_display_ui';
	const SECTION_USAGE = 'usage_section';
	const SECTION_SUPPORT = 'support_section';

	// @since 1.18.0.220401
	const SECTION_COUNTRY_SELECTION = 'country_selection_section';

	
	// @since 1.17.0.210511
	const SECTION_FREEMIUS_ACCOUNT = 'freemius_account_section';
	const SECTION_FREEMIUS_PRICING = 'freemius_pricing_section';
	const SECTION_FREEMIUS_CONTACT = 'freemius_contact_section';
	


	/**
	 * Builds the HTML for the triggers that add countries automatically (e.g. all
	 * the EU countries).
	 *
	 * @return string
	 */
	protected function get_countries_triggers_html(): string {
		$html = '';
		$html .= '<span class="trigger add_eu_countries">';
		$html .= __('Add European Union countries', Definitions::TEXT_DOMAIN);
		$html .= '</span>';

		return $html;
	}

	/**
	 * Returns a list of the options available to display prices (with our
	 * without tax).
	 *
	 * @return array
	 */
	protected function tax_display_options() {
		return array(
			'incl' => __('Including tax', Definitions::TEXT_DOMAIN),
			'excl' => __('Excluding tax', Definitions::TEXT_DOMAIN),
		);
	}

	/**
	 * Returns the tabs to be used to render the Settings page.
	 *
	 * @since 1.18.0.220401
	 */
	protected function get_settings_tabs() {
		return [
			// Tax display settings
			self::TAB_TAX_DISPLAY => [
				'id' => self::TAB_TAX_DISPLAY,
				'label' => __('Tax Display', Definitions::TEXT_DOMAIN),
				'priority' => 100,
			],
			// Tax calculation settings
			self::TAB_TAX_CALCULATION => [
				'id' => self::TAB_TAX_CALCULATION,
				'label' => __('Tax Calculation', Definitions::TEXT_DOMAIN),
				'priority' => 110,
			],
			// Country selection settings
			self::TAB_COUNTRY_SELECTION => [
				'id' => self::TAB_COUNTRY_SELECTION,
				'label' => __('Location Selection', Definitions::TEXT_DOMAIN),
				'priority' => 130,
			],
			// Support
			self::TAB_SUPPORT => [
				'id' => self::TAB_SUPPORT,
				'label' => __('Support', Definitions::TEXT_DOMAIN),
				'priority' => 150,
			],
			
			// Freemius License tab
			self::TAB_LICENSE => [
				'id' => self::TAB_LICENSE,
				'label' => __('License', Definitions::TEXT_DOMAIN),
				'priority' => 160,
			],
			
		];
	}

	/**
	 * Returns the plugin settings sections.
	 *
	 * @since 1.18.0.220401
	 */
	protected function get_settings_sections() {
		$settings_sections = [ // NOSONAR
			self::TAB_TAX_DISPLAY => [
				// Tax display settings section
				[
					'id' => self::SECTION_TAX_DISPLAY,
					'label' => __('Tax Display', Definitions::TEXT_DOMAIN),
					'callback' => array($this, 'tax_display_settings_section_callback'),
					'priority' => 100,
				],
				// User Interface settings section
				[
					'id' => self::SECTION_USER_INTERFACE,
					'label' => __('User Interface', Definitions::TEXT_DOMAIN),
					'callback' => array($this, 'tax_display_ui_settings_section_callback'),
					'priority' => 100,
				],
			],
			self::TAB_TAX_CALCULATION => [
				// Tax calculation settings section
				[
					'id' => self::SECTION_TAX_CALCULATION,
					'label' => __('Tax Calculation', Definitions::TEXT_DOMAIN),
					'callback' => array($this, 'tax_calculation_settings_section_callback'),
					'priority' => 140,
				],
			],
			// @since x,x
			self::TAB_COUNTRY_SELECTION => [
				// Country selection section
				[
					'id' => self::SECTION_COUNTRY_SELECTION,
					'label' => __('Location Selection', Definitions::TEXT_DOMAIN),
					'callback' => array($this, 'country_selection_settings_section_callback'),
					'priority' => 140,
				],
			],
			self::TAB_SUPPORT => [
				// Legacy Licensing feature removed

				
				// Freemius Support section
				// @since 1.18.0.220401
				[
					'id' => self::SECTION_SUPPORT,
					'label' => __('Troubleshooting', Definitions::TEXT_DOMAIN),
					'callback' => array($this, 'freemius_support_section_callback'),
					'priority' => 180,
				],
				// Freemius Contact section
				// @since 1.18.0.220401
				[
					'id' => self::SECTION_FREEMIUS_CONTACT,
					'label' => __('Contact', Definitions::TEXT_DOMAIN),
					'callback' => array($this, 'freemius_contact_section_callback'),
					'priority' => 190,
				],
				
			],
			
			self::TAB_LICENSE => [
				// Account section
				// @since 1.18.0.220401
				[
					'id' => self::SECTION_FREEMIUS_ACCOUNT,
					'label' => __('Account and licence', Definitions::TEXT_DOMAIN),
					'callback' => array($this, 'freemius_account_section_callback'),
					'priority' => 300,
				],
			],
			
		];

		
		// TODO Add rendering of pricing section when the bugs affecting that page are fixed
		// @link https://github.com/Freemius/wordpress-sdk/issues/498
		// $fs_integration = Freemius_Plugin_Integration::get_freemius_integration(Definitions::PLUGIN_SLUG);
		// if(isset($fs_integration) && $fs_integration->is_pricing_page_visible()) {
		// 	$settings_sections[self::TAB_LICENSE][] = [
		// 		'id' => self::SECTION_FREEMIUS_PRICING,
		// 		'label' => __('Plans and pricing', Definitions::TEXT_DOMAIN),
		// 		'callback' => array($this, 'account_pricing_callback'),
		// 		'priority' => 350,
		// 	];
		// }
		


		return $settings_sections;
	}

	/**
	 * Configures the plugin settings fields.
	 *
	 * @return array
	 * @since 1.18.0.220401
	 */
	protected function get_settings_fields(): array { // NOSONAR

		return [
			self::SECTION_TAX_DISPLAY => [
				// "Tax display settings" table
				[
					'id' => Settings::FIELD_TAX_DISPLAY_SETTINGS,
					'label' => wp_kses_post(__('Set the tax display settings for the countries.', Definitions::TEXT_DOMAIN)),
					'description' => '',
					'css_class' => Settings::FIELD_TAX_DISPLAY_SETTINGS,
					'attributes' => [
						'settings_key' => $this->_settings_key,
						'exchange_rates' => $this->current_settings(Settings::FIELD_TAX_DISPLAY_SETTINGS, $this->default_settings(Settings::FIELD_TAX_DISPLAY_SETTINGS, [])),
						'id' => Settings::FIELD_TAX_DISPLAY_SETTINGS,
						'label_for' => Settings::FIELD_TAX_DISPLAY_SETTINGS,
						'attributes' => [
							'class' => Settings::FIELD_TAX_DISPLAY_SETTINGS,
						],
					],
					'type' => 'custom',
					'render_callback' => [$this, 'render_tax_display_settings_options'],
				],
			],
			self::SECTION_TAX_CALCULATION => [
				// Keep product prices fixed
				[
					'id' => Settings::FIELD_KEEP_PRODUCT_PRICES_FIXED,
					'label' => wp_kses_post(__('Keep product price fixed, regardless of what tax rate applies to customer', Definitions::TEXT_DOMAIN)),
					'description' => wp_kses_post(implode(' ', [
						__('Tick this option if you would like to show fixed product prices to your customers, regardless of what tax rate applies to them.', Definitions::TEXT_DOMAIN),
						__('For example, if you set a product price of 100 USD including tax, it will be shown as 100 USD including 10% tax, 20% tax or any tax rate.', Definitions::TEXT_DOMAIN),
						__('The customer will, therefore, always pay 100 USD at the end.', Definitions::TEXT_DOMAIN),
						'<br />',
						'<strong>',
						__('Important', Definitions::TEXT_DOMAIN) . ':',
						'</strong>',
						__('this option is effective only if you enter prices inclusive of tax.', Definitions::TEXT_DOMAIN),
						__('Please go to <code>WooCommerce > Settings > Tax</code> page to change how prices are entered.',	Definitions::TEXT_DOMAIN),
					])),
					'css_class' => '',
					'attributes' => [],
					'type' => 'checkbox',
				],
				// Handle customer's State/Province
				[
					'id' => Settings::FIELD_HANDLE_CUSTOMER_STATE,
					'label' => wp_kses_post(__("Handle customer's State/County/Province", Definitions::TEXT_DOMAIN)),
					'description' => wp_kses_post(implode(' ', [
						__("Tick this option if you would like the Tax Display plugin to take visitor's State/County/Province into account.", Definitions::TEXT_DOMAIN),
						__('This setting will show the "State/County" field on the country selector widget, allowing customers to select their State/County, as well as their country.',
								Definitions::TEXT_DOMAIN),
						'<br />',
						'<strong>',
						__('Note', Definitions::TEXT_DOMAIN) . ':',
						'</strong>',
						__('when this option is enabled, the country selector widget will no longer automatically reload the page when the user selects a country.', Definitions::TEXT_DOMAIN),
						__('The "change location" button will be displayed instead, to allow the customer to select both the country and the State before confirming the selection.',
								Definitions::TEXT_DOMAIN),
					])),
					'css_class' => '',
					'attributes' => [],
					'type' => 'checkbox',
				],
			],
			self::SECTION_USER_INTERFACE => [
				// Display the "show me prices without tax" checkbox on the country selector widget
				[
					'id' => Settings::FIELD_TAX_EXEMPT_FLAG_ENABLED,
					'label' => wp_kses_post(__('Allow users to indicate that they are exempt from tax/VAT', Definitions::TEXT_DOMAIN)),
					'description' => wp_kses_post(implode(' ', [
						__('When enabled, this option will display a checkbox on the country selector widget, which visitors can tick if they are exempt from tax/VAT',
								Definitions::TEXT_DOMAIN),
						__('By doing so, they will see prices without tax, regardless of any other settings.', Definitions::TEXT_DOMAIN),
						'<br />',
						'<strong>',
						__('Important', Definitions::TEXT_DOMAIN) . ':',
						'</strong>',
						__('this setting is purely cosmetic.', Definitions::TEXT_DOMAIN),
						__('To ensure that taxes are not applied at checkout to customers who are exempt, you will have to use a plugin that can handle such an exemption.',
								Definitions::TEXT_DOMAIN),
					])),
					'css_class' => '',
					'attributes' => [],
					'type' => 'checkbox',
				],
				// Label for the "show me prices without tax" checkbox
				[
					'id' => Settings::FIELD_TAX_EXEMPT_FLAG_LABEL,
					'label' => wp_kses_post(__('"I\'m tax exempt" widget label', Definitions::TEXT_DOMAIN)),
					'description' => wp_kses_post(implode(' ', [
						__('This label is displayed on the country selector widget, next to the checkbox that visitors can select to see prices without tax/VAT.',
								Definitions::TEXT_DOMAIN),
					])),
					'css_class' => '',
					'attributes' => [
						'placeholder' => $this->default_settings(Settings::FIELD_TAX_EXEMPT_FLAG_LABEL, ''),
					],
					'type' => 'text',
				],
				// Price suffix when the "show me prices without VAT" is enabled
				[
					'id' => Settings::FIELD_TAX_EXEMPT_PRICE_SUFFIX,
					'label' => wp_kses_post(__('"I\'m tax exempt" price suffix', Definitions::TEXT_DOMAIN)),
					'description' => wp_kses_post(implode(' ', [
						__('This price suffix is be displayed to visitors who choose to see prices without tax/VAT.', Definitions::TEXT_DOMAIN),
					])),
					'css_class' => '',
					'attributes' => [
						'placeholder' => __('Use default', Definitions::TEXT_DOMAIN),
					],
					'type' => 'text',
				],
			],
			// @since 1.18.0.220401
			self::SECTION_COUNTRY_SELECTION => [
				// Option to reset the country upon login
				[
					'id' => Settings::FIELD_USE_LOCATION_FROM_PROFILE_ON_LOGIN,
					'label' => wp_kses_post(__("Use location from user's profile upon login", Definitions::TEXT_DOMAIN)),
					'description' => wp_kses_post(implode(' ', [
						__('When enabled, the option fetches the location from the user profile when a visitor logs in.', Definitions::TEXT_DOMAIN),
						__('In this case, the location loaded from the profile replaces any previous selection, immediately after the login.', Definitions::TEXT_DOMAIN),
						__('Users can change the location again (country and state), using the country selector widget, or at checkout.', Definitions::TEXT_DOMAIN),
					])),
					'css_class' => '',
					'attributes' => [],
					'type' => 'checkbox',
				],
			],
		];
	}

	/**
	 * Returns the title for the menu item that will bring to the plugin's
	 * settings page.
	 *
	 * @return string
	 */
	protected function menu_title() {
		return __('Tax Display by Country', Definitions::TEXT_DOMAIN);
	}

	/**
	 * Returns the slug for the menu item that will bring to the plugin's
	 * settings page.
	 *
	 * @return string
	 */
	protected function menu_slug() {
		return Definitions::MENU_SLUG;
	}

	/**
	 * Returns the title for the settings page.
	 *
	 * @return string
	 */
	protected function page_title() {
		return __('Tax Display by Country - Settings', Definitions::TEXT_DOMAIN) .
					 sprintf('&nbsp;(v. %s)', WC_Aelia_Tax_Display_By_Country::$version);
	}

	/**
	 * Returns the description for the settings page.
	 *
	 * @return string
	 */
	protected function page_description() {
		return __('In this page you can configure the settings for the Tax Display by ' .
							'Country plugin. ' .
							'Using the interface below, you will be able to specify how prices will ' .
							'be displayed, based on customer\'s country.',
							Definitions::TEXT_DOMAIN);
	}

	/*** Settings sections callbacks ***/

	/**
	 * Renders the tax display settings section.
	 *
	 * @return void
	 */
	public function tax_display_settings_section_callback(): void {
		echo wp_kses_post(implode(' ', [
			__('In this section you can configure how the prices will be displayed (with our without taxes) for the various countries.', Definitions::TEXT_DOMAIN),
			__('You can reorder the settings by dragging the handle on the left.', Definitions::TEXT_DOMAIN),
			__('Please keep in mind that, if you add the same country to multiple rows, the settings will be taken <u>from the first matching entry</u>.', Definitions::TEXT_DOMAIN),
			'<noscript>',
			__('This page requires JavaScript to work properly. Please enable JavaScript in your browser and refresh the page.</u>.', Definitions::TEXT_DOMAIN),
			'</noscript>',
		]));
	}

	/**
	 * Renders the tax calculations settings section.
	 *
	 * @return void
	 */
	public function tax_calculation_settings_section_callback(): void {
		echo wp_kses_post(implode(' ', [
			__('In this section you can configure settings that alter how the tax will be calculated.', Definitions::TEXT_DOMAIN),
			'<div class="important note">',
			'<strong>',
			__('Important', Definitions::TEXT_DOMAIN),
			':</strong>',
			__('Make sure that you are allowed to alter the default tax calculations', Definitions::TEXT_DOMAIN),
			__('It will be your responsibility to be compliant with the tax laws that apply in your country.', Definitions::TEXT_DOMAIN),
			__('We will not be able to provide assistance in verifying that the taxes are being calculated correctly, according to your country\'s laws.', Definitions::TEXT_DOMAIN),
			__('We recommend that you talk to your accountant before changing these settings.',	Definitions::TEXT_DOMAIN),
			'</div>',
		]));
	}

	/**
	 * Renders the country selection settings section.
	 *
	 * @return void
	 */
	public function country_selection_settings_section_callback(): void {
		echo wp_kses_post(implode(' ', [
			__('In this section you can find settings that can alter the country selection logic will work.', Definitions::TEXT_DOMAIN),
		]));
	}

	/**
	 * Renders the tax display UI settings section.
	 *
	 * @return void
	 */
	public function tax_display_ui_settings_section_callback(): void {
		echo wp_kses_post(implode(' ', [
			__('In this section you can configure the settings related to the user interface.', Definitions::TEXT_DOMAIN),
		]));
	}

	// Legacy Licensing feature removed

	
	/**
	 * Support section (Freemius)
	 *
	 * @return void
	 * @since 1.17.0.210511
	 */
	public function freemius_support_section_callback() {
		?>
		<div class="troubleshooting">
			<p><?php
				echo wp_kses_post(implode(' ', array(
					__('We designed our solutions to be robust and effective, as well as intuitive and easy to use.', Definitions::TEXT_DOMAIN),
					__('The options in this section can help you troubleshooting any issue that you might encounter while using our solution.', Definitions::TEXT_DOMAIN),
				)));
			?></p>
		</div>
		<?php
	}

	/**
	 * Freemius Contact page.
	 *
	 * @return void
	 * @since 1.17.0.210511
	 */
	public function freemius_contact_section_callback() {
		?>
		<div class="support_information">
			<p><?php
				echo wp_kses_post(__('Should you need assistance, or if you just would like to get in touch with us, please feel free to use the form below.', Definitions::TEXT_DOMAIN));
			?></p>
			<div class="contact_form"><?php
				echo do_shortcode(sprintf('[aelia_freemius_contact_form plugin_slug="%1$s"]', Definitions::PLUGIN_SLUG));
			?></div>
		</div>
		<?php
	}

	/**
	 * Account section. Shows the Freemius account details.
	 *
	 * @return void
	 * @since 1.17.0.210511
	 */
	public function freemius_account_section_callback(): void {
		?>
		<div class="freemius_account"><?php
		if(Freemius_Plugin_Integration::is_plugin_license_activated(Definitions::PLUGIN_SLUG)) {
			echo implode(' ', array(
				__('This plugin is distributed via the Freemius platform, which also handles the customer accounts and the licences.', Definitions::TEXT_DOMAIN),
				sprintf(
					__('You can review the account settings for this plugin <a class="freemius_account_link" href="%1$s" target="_blank">in the dedicated account page</a>.', Definitions::TEXT_DOMAIN),
					Freemius_Plugin_Integration::get_account_page_url(Definitions::PLUGIN_SLUG)
				),
			));
		}
		else {
			$fs_integration = Freemius_Plugin_Integration::get_freemius_integration(Definitions::PLUGIN_SLUG);
			echo implode(' ', [
				__('It looks like this site is not covered by an active licence.', Definitions::TEXT_DOMAIN),
				sprintf(
					__('Please <a class="freemius_activation_link" href="%1$s">complete the activation</a> to receive updates and support for the product.', Definitions::TEXT_DOMAIN),
					$fs_integration->get_activation_url(array(), !$fs_integration->is_delegated_connection())
				),
			]);
		}
		?></div>
		<?php
	}

	/**
	 * Freemius Pricing section. Shows the pricing options for a plugin.
	 *
	 * @return void
	 * @since 1.17.0.210511
	 */
	public function account_pricing_callback(): void {
		?>
		<div class="freemius_pricing_options"><?php
			echo do_shortcode(sprintf('[aelia_freemius_plugin_pricing plugin_slug="%1$s"]', Definitions::PLUGIN_SLUG));
		?></div>
		<?php
	}
	

	/*** Rendering methods ***/
	/**
	 * Renders a table containing several fields that Admins can use to configure
	 * how prices will be displayed for visitors from various countries.
	 *
	 * @param array args An array of arguments passed by add_settings_field().
	 * @see add_settings_field().
	 */
	public function render_tax_display_settings_options($args) {
		global $woocommerce;

		function render_actions_column($textdomain) {
			$result = '<td class="actions">';
			$result .= '<a href="#" class="button minus remove">';
			$result .= __('Remove', $textdomain);
			$result .= '</a>';
			$result .=  '</td>';

			return $result;
		}

		// Note
		// Variables $tax_display_field_id and $tax_display_field_name are output variables. They
		// are deliberately null at this point, they will be set by the method
		$tax_display_field_id = null;
		$tax_display_field_name = null;
		$this->get_field_ids($args, $tax_display_field_id, $tax_display_field_name);

		// Add elements that can be cloned as new rows, with all the required fields
		$html = '<table id="tax_display_settings">';
		// Table header
		$html .= '<thead>';
		$html .= '<tr>';
		$html .= '<td class="sort">&nbsp;</td>';
		$html .= '<td>' . __('Countries', Definitions::TEXT_DOMAIN) . '</td>';
		$html .= '<td class="cart_prices">' . __('Show cart prices', Definitions::TEXT_DOMAIN) . '</td>';
		$html .= '<td class="shop_prices">' . __('Show shop prices', Definitions::TEXT_DOMAIN) . '</td>';
		$html .= '<td class="price_suffix">' . __('Price suffix', Definitions::TEXT_DOMAIN) . '</td>';

		$html .= '<td class="actions">&nbsp;</td>';
		$html .= '</tr>';
		$html .= '</thead>';
		$html .= '<tbody>';

		// Template row
		$html .= '<tr class="template">';
		$html .= '<td class="sort handle">&nbsp;</td>';
		// Render countries list field
		$html .= '<td class="countries">';
		$field_args = array(
			'id' => $tax_display_field_id . '_X[countries]',
			'name' => $tax_display_field_name . '[X][countries]',
			'selected' => array(),
			'options' => $woocommerce->countries->get_allowed_countries(),
			'attributes' => array(
				'class' => 'input',
				'multiple' => 'multiple',
			),
		);
		ob_start();
		$this->render_dropdown($field_args);
		$field_html = ob_get_contents();
		ob_end_clean();
		$html .= $field_html;

		// Add the triggers that allow to add sets of countries in one shot
		$html .= '<div class="triggers">';
		$html .= $this->get_countries_triggers_html();
		$html .= '</div>';

		$html .= '</td>';

		// Render the "cart prices" tax display field
		$html .= '<td class="cart_prices">';
		$field_args = array(
			'id' => $tax_display_field_id . '_X[cart_prices]',
			'name' => $tax_display_field_name . '[X][cart_prices]',
			'selected' => 'incl',
			'options' => $this->tax_display_options(),
			'attributes' => array(
				'class' => 'input tax_display_type',
			),
		);
		ob_start();
		$this->render_dropdown($field_args);
		$field_html = ob_get_contents();
		ob_end_clean();
		$html .= $field_html;
		$html .= '</td>';

		// WooCommerce 2.1+ settings
		if(version_compare($woocommerce->version, '2.1', '>=')) {
			// Render the "shop prices" tax display field
			$html .= '<td class="shop_prices">';
			$field_args = array(
				'id' => $tax_display_field_id . '_X[shop_prices]',
				'name' => $tax_display_field_name . '[X][shop_prices]',
				'selected' => 'incl',
				'options' => $this->tax_display_options(),
				'attributes' => array(
					'class' => 'input tax_display_type',
				),
			);
			ob_start();
			$this->render_dropdown($field_args);
			$field_html = ob_get_contents();
			ob_end_clean();
			$html .= $field_html;
			$html .= '</td>';

			// Render the "price suffix" field
			$html .= '<td class="price_suffix">';
			$field_args = array(
				'id' => $tax_display_field_id . '_X[price_suffix]',
				'name' => $tax_display_field_name . '[X][price_suffix]',
				'value' => '',
				'attributes' => array(
					'class' => 'input price_suffix',
					'placeholder' => __('Use default', Definitions::TEXT_DOMAIN),
				),
			);
			ob_start();
			$this->render_textbox($field_args);
			$field_html = ob_get_contents();
			ob_end_clean();
			$html .= $field_html;
			$html .= '</td>';
		}
		// Render action column
		$html .= render_actions_column(Definitions::TEXT_DOMAIN);
		$html .= '</tr>';
		// Template row - End

		$tax_display_settings = $this->current_settings('tax_display_settings');

		if(!is_array($tax_display_settings)) {
			$tax_display_settings = array();
		}

		foreach($tax_display_settings as $index => $settings) {
			$html .= '<tr class="data">';
			$html .= '<td class="sort">&nbsp;</td>';

			// Render countries list field
			$html .= '<td class="countries">';
			$field_args = array(
				'id' => $tax_display_field_id . '_' . $index . '[countries]',
				'name' => $tax_display_field_name . '[' . $index . '][countries]',
				'selected' => get_value('countries', $tax_display_settings[$index], array()),
				'options' => $woocommerce->countries->get_allowed_countries(),
				'attributes' => array(
					'class' => 'input',
					'multiple' => 'multiple',
				),
			);
			ob_start();
			$this->render_dropdown($field_args);
			$field_html = ob_get_contents();
			ob_end_clean();
			$html .= $field_html;

			// Add the triggers that allow to add sets of countries in one shot
			$html .= '<div class="triggers">';
			$html .= $this->get_countries_triggers_html();
			$html .= '</div>';

			$html .= '</td>';

			// Render the "cart prices" tax display field
			$html .= '<td class="cart_prices">';
			$field_args = array(
				'id' => $tax_display_field_id . '_' . $index . '[cart_prices]',
				'name' => $tax_display_field_name . '[' . $index . '][cart_prices]',
				'selected' => get_value('cart_prices', $tax_display_settings[$index], array()),
				'options' => $this->tax_display_options(),
				'attributes' => array(
					'class' => 'input tax_display_type',
				),
			);
			ob_start();
			$this->render_dropdown($field_args);
			$field_html = ob_get_contents();
			ob_end_clean();
			$html .= $field_html;
			$html .= '</td>';

			// WooCommerce 2.1+ settings
			if(version_compare($woocommerce->version, '2.1', '>=')) {
				// Render the "shop prices" tax display field
				$html .= '<td class="shop_prices">';
				$field_args = array(
					'id' => $tax_display_field_id . '_' . $index . '[shop_prices]',
					'name' => $tax_display_field_name . '[' . $index . '][shop_prices]',
					'selected' => get_value('shop_prices', $tax_display_settings[$index], array()),
					'options' => $this->tax_display_options(),
					'attributes' => array(
						'class' => 'input tax_display_type',
					),
				);
				ob_start();
				$this->render_dropdown($field_args);
				$field_html = ob_get_contents();
				ob_end_clean();
				$html .= $field_html;
				$html .= '</td>';

				// Render the "price suffix"  field
				$html .= '<td class="price_suffix">';
				$field_args = array(
					'id' => $tax_display_field_id . '_' . $index . '[price_suffix]',
					'name' => $tax_display_field_name . '[' . $index . '][price_suffix]',
					'value' => get_value('price_suffix', $tax_display_settings[$index], ''),
					'attributes' => array(
						'class' => 'input price_suffix',
						'title' => __('This price suffix will override the one configured by default in ' .
													'WooCommerce settings. Leave it empty to use the default one.',
													Definitions::TEXT_DOMAIN),
						'placeholder' => __('Use default', Definitions::TEXT_DOMAIN),
					),
				);
				ob_start();
				$this->render_textbox($field_args);
				$field_html = ob_get_contents();
				ob_end_clean();
				$html .= $field_html;
				$html .= '</td>';			}
			// Render action column
			$html .= render_actions_column(Definitions::TEXT_DOMAIN);
			$html .= '</tr>';
		}
		$html .= '</tbody>';

		// Table footer
		$html .= '
			<tfoot>
				<tr>
					<th colspan="5">
						<a href="#" class="button plus insert">Insert row</a>
					</th>
				</tr>
			</tfoot>
		';
		$html .= '</table>';

		echo $html;
	}
}
